package premium_store;

import premium_store.model.Client;
import premium_store.model.Level;
import premium_store.model.Nation;
import premium_store.model.Tank;
import premium_store.service.ClientService;
import premium_store.service.LevelService;
import premium_store.service.NationService;
import premium_store.service.TankService;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import javax.persistence.EntityNotFoundException;
import java.util.*;

@SpringBootTest
class PremiumStoreApplicationTests {
    private static final Logger log = LoggerFactory.getLogger(PremiumStoreApplicationTests.class);

	@Autowired
    TankService tankService;

    @Autowired
    ClientService clientService;

	@Autowired
    LevelService levelService;

	@Autowired
    NationService nationService;

    @Test
    void testClientRead() {
        clientService.deleteAllClients();

        Client client = clientService.addClient("3tankista73", "fff@mail.ru", 3400);
        log.info(client.toString());

        Client findClient = clientService.findClient(client.getId());
        log.info(findClient.toString());

        Assertions.assertEquals(client, findClient);
    }

	@Test
	void testNationRead() {
		nationService.deleteAllNations();
		tankService.deleteAllTanks();
		levelService.deleteAllLevels();

		Nation nation = nationService.addNation("Германия");

		Level firstLevel = levelService.addLevel(8);
		Level secondLevel = levelService.addLevel(9);

		Tank firstTank =  tankService.addTank("ИС-3", nation, firstLevel, 3700000);
		Tank secondTank = tankService.addTank("E-75", nation, secondLevel, 5600000);

		List<Tank> tanks = new ArrayList<>(Arrays.asList(firstTank, secondTank));

		nation = nationService.updateNation(nation.getId(), "СССР", tanks);

		log.info(nation.toString());

		Nation findNation = nationService.findNation(nation.getId());
		log.info(findNation.toString());

		Assertions.assertEquals(nation, findNation);
	}

	@Test
	void testClientReadNotFound() {
		clientService.deleteAllClients();
		Assertions.assertThrows(EntityNotFoundException.class, () -> clientService.findClient(-1L));
	}

	@Test
	void testLevelRead() {
		tankService.deleteAllTanks();
		levelService.deleteAllLevels();

		Level level = levelService.addLevel(8);
		log.info(level.toString());

		Level secondLevel = levelService.addLevel(9);
		log.info(secondLevel.toString());

		Assertions.assertEquals(levelService.findAllLevels().size(), 2);
	}

	@Test
	void testLevelReadAllEmpty() {
		tankService.deleteAllTanks();
		levelService.deleteAllLevels();

		List<Level> levels = levelService.findAllLevels();
		log.info(levels.toString());

		Assertions.assertEquals(levels.size(), 0);
	}

	@Test
	void testTankReadAll() {
		tankService.deleteAllTanks();
        levelService.deleteAllLevels();

		Level firstLevel = levelService.addLevel(8);
		Level secondLevel = levelService.addLevel(9);

		tankService.addTank("ИС-3", nationService.addNation("СССР"), firstLevel, 3700000);
		tankService.addTank("E-75", nationService.addNation("Германия"), secondLevel, 5600000);

		List<Tank> tanks = tankService.findAllTanks();
		log.info(tanks.toString());

		Assertions.assertEquals(tanks.size(), 2);
	}

	@Test
	void testClientCreate() {
		clientService.deleteAllClients();

		Client firstClient = clientService.addClient("Barbarian", "dsfg@gmail.com", 56000);
		log.info(firstClient.toString());

		Client secondClient = clientService.addClient("KorbenDallas", "tankoviyGeniy@mail.ru", 37000);
		log.info(secondClient.toString());

		Assertions.assertEquals(clientService.findAllClients().size(), 2);
	}

	@Test
	void testUpdateTank(){
        tankService.deleteAllTanks();
        levelService.deleteAllLevels();
		clientService.deleteAllClients();

		Level firstLevel = levelService.addLevel(8);
		Level secondLevel = levelService.addLevel(9);

		Tank firstTank =  tankService.addTank("ИС-3", nationService.addNation("СССР"), firstLevel, 3700000);
		Tank secondTank = tankService.addTank("E-75", nationService.addNation("Германия"), secondLevel, 5600000);

		Tank tank = tankService.findTank(firstTank.getId());
		log.info(tank.toString());

		tank.setName("ИСУ-152");
		tank.setLevel(secondLevel);
		tank.setCost(4100000);

		log.info(tank.toString());
	}

	@Test
	void testNewTest() {
		nationService.deleteAllNations();
		tankService.deleteAllTanks();
		levelService.deleteAllLevels();

		Nation nation = nationService.addNation("Германия");
		Nation newNation = nationService.addNation("CCCР");

		Level firstLevel = levelService.addLevel(8);
		Level secondLevel = levelService.addLevel(9);

		Tank firstTank =  tankService.addTank("ИС-3", nation, firstLevel, 3700000);
		Tank secondTank = tankService.addTank("E-75", nation, secondLevel, 5600000);

		List<Tank> tanks = new ArrayList<>(Arrays.asList(firstTank, secondTank));

		nation = nationService.updateNation(nation.getId(), "СССР", tanks);

		tanks = tankService.findTank(firstLevel, nation);

		log.info("AAAAA" + tanks.toString());
	}
}
